/*
 *				**** ALBLIST ****
 *
 * This file contains routines that read the
 * alblist.txt and songlist.txt files, and
 * stores them for use by calling routines
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "alblist.h"

static int verbose = 5;	/* verbose switch */
static char *Prog=NULL;		/* program name */
static int ncells=4;	/* how many albums per row */

static ProdPtrList *AlbCat = NULL;	/* album list by categories */
static ProdAPtrList *AlbKey = NULL;	/* album index by key */
static ProdAPtrList *AlbTitle = NULL;	/* album index by title */

static void read_alblist(void);	/* ingest alblist.txt */
static struct album *create_album(void);
static struct albcat *create_albcat(void);



void
alblist(char *pname)
{
	Prog = pname;
   if (verbose > 5) printf("%s: Initializing alblist\n",
   	Prog);

	read_alblist();	/* ingest alblist.txt */
   if (verbose > 5) printf("%s: %d categories, %d albums read\n",
   							Prog, AlbCat->nitems, AlbTitle->nitems);
}

void
set_verbose_alblist(int v)
{
	verbose = v;
}

int get_number_albums(void)
{
	if (AlbTitle == NULL) return -1;
	return AlbTitle->nitems;
}

char **get_album_titles(void)
{
	if (AlbTitle == NULL) return NULL;
	return AlbTitle->key;
}

char **get_album_keys(void)
{
	if (AlbKey == NULL) return NULL;
   return AlbKey->key;
}

static void
read_alblist(void)
{
	FILE *fp;
   char buf[BUFSIZ], *p;
   struct albcat *cat = NULL;
   struct album *alb;

	if (AlbCat != NULL)
   {
   	fprintf(stderr, "%s: read_alblist called twice\n", Prog);
		return;
   }
   AlbCat = create_ptr_prodlist(10);	/* max 10 categories */
   AlbKey = create_aptr_prodlist(50);	/* max 50 albums */
   AlbTitle = create_aptr_prodlist(50);	/* max 50 albums */

   if ((fp = fopen("alblist.txt", "r")) == NULL)
   {
   	fprintf(stderr, "%s: Can't open alblist.txt\n", Prog);
      return;
   }

   while (read_line(buf, fp) == 0)
   { /* read lines and store in AlbCat */
   	if (*buf == '\0') continue;	/* blank line */
   	if (*buf == '*')
      { /* new category */
      	cat = create_albcat();
         if (cat == NULL)
         {
         	fprintf(stderr, "%s: Can't malloc for albcat!\n", Prog);
            return;
         }
         cat->desc = strdup(buf+1);	/* category description */
         cat->list = create_ptr_prodlist(25);	/* albums */
      	add_ptr_prodlist(AlbCat, cat);	/* add category */
         continue;	/* read next line */
		}
      else if (cat == NULL)
      { /* no category at start of file */
      	fprintf(stderr, "%s: alblist.txt must start with category!\n",
         	Prog);
         return;
      }

      /* line is:  title | key */
      for (p=buf; *p != '\0' && *p != '|'; p++);
      if (*p == '\0')
      { /* bad entry */
			fprintf(stderr, "%s: Album missing key (%s)\n", Prog, buf);
         continue;	/* ignore entry */
      }
      *p++ = '\0';	/* split into two fields */
      alb = create_album();
      if (alb == NULL)
      {
      	fprintf(stderr, "%s: Can't malloc for album (%s)\n",
         	Prog, buf);
         return;
      }
      alb->key = strdup(p);	/* album key */
      alb->title = strdup(buf);	/* album title */
      add_ptr_prodlist(cat->list, (void *)alb);	/* add to category */
      /*  add to album index */
      bupdate_aptr_prodlist(AlbKey, alb->key, alb);
      bupdate_aptr_prodlist(AlbTitle, alb->title, alb);
   }
}

struct album *
create_album(void)
{
	struct album *ret;

   ret = (struct album *)malloc(sizeof(struct album));
   if (ret == NULL) return NULL;
   memset(ret, '\0', sizeof(struct album)); /* clear */
   return ret;
}

struct albcat *
create_albcat(void)
{
	struct albcat *ret;

   ret = (struct albcat *)malloc(sizeof(struct albcat));
   if (ret == NULL) return NULL;
   memset(ret, '\0', sizeof(struct albcat)); /* clear */
   return ret;
}

void
display_alblist(FILE *fp)
{
   int i, j;
   struct album *alb;
   struct albcat *cat;

   fprintf(fp, "%d albums, %d categories\n",
   	AlbKey->nitems, AlbCat->nitems);
   for (i=0; i < AlbCat->nitems; i++)
   {
   	cat = (struct albcat *)AlbCat->item[i];
      fprintf(fp, "*** %s\n", cat->desc);
      for (j=0; j < cat->list->nitems; j++)
      {
      	alb = (struct album *)cat->list->item[j];
         fprintf(fp, "\t%s\t%s\n", alb->key, alb->title);
      }
   }
}

void
create_discog(FILE *fp)
{
	int ncat, nalb;
   struct albcat *albcat;
   struct album *alb;

   /* generate title and top of page */
   fprintf(fp, "Discography\n\n");	/* title */

   /* create link for each category */
   fprintf(fp, "<strong>\n");
   for (ncat=0; ncat < AlbCat->nitems; ncat++)
   {
   	albcat = (struct albcat *)AlbCat->item[ncat]; /* category */
      fprintf(fp, "<a href=#cat%d><br>%s</a>\n", ncat, albcat->desc);
   }
   fprintf(fp, "</strong><p>\n");

   /* create actual entries */
   for (ncat=0; ncat < AlbCat->nitems; ncat++)
   { /* loop thru categories */
   	albcat = (struct albcat *)AlbCat->item[ncat]; /* category */
      fprintf(fp, "<a name=cat%d><h2><center>%s</center></h2></a>\n<ul>\n",
      	ncat, albcat->desc);
      fprintf(fp, "<table border=2 align=center>\n");
		for (nalb=0; nalb<albcat->list->nitems; nalb++)
      { /* loop thru albums in categories */
      	/* check for start of row */
      	if ((nalb%ncells) == 0) fprintf(fp, "<tr>\n");
      	alb = (struct album *)albcat->list->item[nalb]; /* album */
         fprintf(fp, "<td align=center>\n");
         fprintf(fp, "<a href=../discog/%s.html#beg>\n", alb->key);
         fprintf(fp, "%s</td></a>\n", alb->title);
         if ((nalb%ncells) == (ncells-1)) fprintf(fp, "</tr>\n");
      }
      /* finish up last row if needed */
      /* for (;(nalb%ncells) > 0; nalb++) fprintf(fp, "<td></td>"); */
      fprintf(fp, "</tr></table>\n");
      fprintf(fp, "</ul>\n>Top\n");
   }
}

void
create_discog_links(FILE *fp)
{
	int ncat, nalb;
   struct albcat *albcat;
   struct album *alb;

   /* generate table of contents at the top of the page */
   fprintf(fp, "Discography Links\n\n"); /* title of page */

   /* create link for each category */
   for (ncat=0; ncat < AlbCat->nitems; ncat++)
   {
   	albcat = (struct albcat *)AlbCat->item[ncat]; /* category */
      fprintf(fp, "<a href=#cat%d><li>%s</a><p>\n", ncat, albcat->desc);
   }

   /* create actual entries */
   for (ncat=0; ncat < AlbCat->nitems; ncat++)
   { /* loop thru categories */
   	albcat = (struct albcat *)AlbCat->item[ncat]; /* category */
      fprintf(fp, "<a name=cat%d><h4>%s</h4></a>\n",
      	ncat, albcat->desc);
		for (nalb=0; nalb<albcat->list->nitems; nalb++)
      { /* loop thru albums in categories */
      	alb = (struct album *)albcat->list->item[nalb]; /* album */
         fprintf(fp, "<a href=../discog/%s.html#beg target=det><li>%s</a><p>\n",
         	alb->key, alb->title);
      }
      fprintf(fp, ">Top\n");
   }
}

struct album *
find_album_by_title(char *title)
{
	int loc;

   loc = bsearch_aptr_prodlist(AlbTitle, title);
   if (loc < 0) return NULL;
   return (struct album *)AlbTitle->item[loc];
}

struct album *
find_album_by_key(char *key)
{
	int loc;

   loc = bsearch_aptr_prodlist(AlbKey, key);
   if (loc < 0) return NULL;
   return (struct album *)AlbKey->item[loc];
}

struct album *
get_album_by_key_loc(int loc)
{
	if (loc < 0 || loc > AlbKey->nitems) return NULL;
   return (struct album *)AlbKey->item[loc];
}

int
is_alb_key(char *key)
{
	int loc;

   loc = bsearch_aptr_prodlist(AlbKey, key);
   return (loc >= 0);
}

char *
get_alb_title_for_key(char *key)
{
	struct album *alb;

   alb = find_album_by_key(key);
   if (alb == NULL) return NULL;
   return alb->title;
}

char *
get_alb_key_for_title(char *title)
{
	struct album *alb;

   alb = find_album_by_title(title);
   if (alb == NULL) return NULL;
   return alb->key;
}

void
alb_tracks(char *prog)
{
	int i;

	Prog = prog;

   for (i=0; i < AlbKey->nitems; i++)
   {
   	read_alb_tracks(prog, (struct album *)AlbKey->item[i]);
   }
}


