#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "bld_html.h"
#include "prodlist.h"

static char *Help[] = {
"\nThis program will build HTML pages.\n",
"The program will read files from the input directory,\n",
"merge them with any include segments that are present,\n",
"and write the resulting page to the output directory.\n",
"The name of the output file will be the same as the input file.\n",
"Therefore, either the input or output directory must be specified.\n",
"\t(i.e. you cannot default both directories)\n",
"If a directory is not specified, then the current directory is assumed.\n",
"\n",
"An include segment is signified with a line:\n",
"\t>segname\n",
"where 'segname' is the name of the include segment.\n",
NULL};

static char *Usage[] = {
"Usage:\t%s [options] files ...\n",
"\t%s [-a] [options]\n",
"-OR- for help, type:\n",
"\t%s -h\n",
"where options are:\n"
"\t[-i indir]\tinput directory\n",
"\t[-o outdir]\toutput directory\n",
"\t[-v val]\tset verbose switch to 'val'\n",
"\t-a\t\tprocess all .html files in indir\n",
NULL};


char *Prog = NULL;	/* program name */
int debugsw = 0;		/* debug switch */
int verbose = 1;		/* verbose switch */
static char *indir=NULL, *outdir=NULL;

void do_msg(char *msg[], char *prog);
void bld_page(char *fname);
void bld_all_page();
ProdAlphaList *list_files_in_directory(char *path);
int check_dir(void);

int
main(int argc, char *argv[])
{
	char buf[BUFSIZ], *p;

   p = strrchr(argv[0], '/');
   if (p == NULL) p = strrchr(argv[0], '\\');
   if (p == NULL) p = argv[0];
   else p++; /* skip past last directory slash */

	Prog = p;	/* save program name */
   /* convert name to lower case & drop .exe (for DOS) */
   while (*p != '\0')
   { /* loop through program name */
   	if (stricmp(p, ".exe") == 0)
      { /* found .EXE (dos) */
      	*p = '\0';
         break;
      }
   	if (isupper(*p)) *p = tolower(*p);
      p++;
   }
   argc--; argv++;	/* skip past program name */

   if (argc == 0)
   {
   	do_msg(Usage, Prog);
   	do_msg(Help, Prog);
      exit(0);
   }

   set_verbose_inc(verbose); /* set verbose flag */

   /* parse the include files */
   bld_inc("include/include.txt");

   while (argc > 0)
   { /* process arguments */
   	if (argv[0][0] == '-') switch(argv[0][1])
      { /* process option */
      	case 'a': bld_all_page(); break;
         case 'd': debugsw = !debugsw; break;
      	case 'h': do_msg(Usage, Prog); do_msg(Help, Prog); exit(0);
         case 'i': /* input directory */
				if (argv[0][2] == '\0')
            { /* input directory is next argument */
            	argc--; argv++;	/* skip -i arg */
               indir = argv[0];	/* output dir */
            }
            else
            { /* input directory follows -o */
            	indir = argv[0]+2;
            }
            break;
         case 'o': /* output directory */
				if (argv[0][2] == '\0')
            { /* output directory is next argument */
            	argc--; argv++;	/* skip -o arg */
               outdir = argv[0];	/* output dir */
            }
            else
            { /* output directory follows -o */
            	outdir = argv[0]+2;
            }
            break;
         case 'v':
         	if (argv[0][2] == '\0')
            { /* verbose value is next arg */
            	argc--; argv++;
               p = argv[0];
            } else p = argv[0]+2;
         	verbose = atoi(p);	/* set verbose flag */
            set_verbose_inc(verbose);
         	break;
         default:
         	fprintf(stderr, "%s: Invalid option (%s)\n",
            	Prog, argv[0]);
            do_msg(Usage, Prog);
            exit(1);
      } else
      { /* not option - must be file name */
      	if (verbose>1) printf("%s: Processing file %s ...\n",
         	Prog, argv[0]);
         strcpy(buf, argv[0]);	/* copy name */

         /* strip .html if there */
         if ((p = strstr(argv[0], ".htm")) != NULL)
         { /* found .htm or .html - get rid of it */
         	*p = '\0';
         }

         /* strip directory name if there */
         p = strrchr(argv[0], '/');
         if (p == NULL) p = strrchr(argv[0], '\\');
         if (p == NULL) p = buf;
         else p++;	/* skip past final slash */
         bld_page(p);	/* build album HTML file */
      }
   	argc--; argv++;	/* done with argument */
   }

   printf("%s: Terminating\n", Prog);
   exit(0);
}

void
bld_page(char *fname)
{
	char buf[BUFSIZ];
   FILE *outf, *inf;	/* output & input files */
   char *p;

	if (verbose > 0) printf("Building page %s ...\n", fname);

   if (check_dir()) return;

   /* strip .html if present */
   if ((p = strstr(fname, ".html")) != NULL) *p = '\0';

   /* try to open input file */
   sprintf(buf, "%s/%s.html", indir, fname);
   if ((inf = fopen(buf, "r")) == NULL)
   {
   	fprintf(stderr, "%s: Can't open input file (%s)!\n",
      	Prog, buf);
      return;
   }

   /* now try output file */
   sprintf(buf, "%s/%s.html", outdir, fname);
   if ((outf = fopen(buf, "w")) == NULL)
   {
   	fprintf(stderr, "%s: Can't open output file (%s)!\n",
      	Prog, buf);
      fclose(inf);
      return;
   }

   /*
    * first line is title
    * Remaining lines up to first non-blank
    * are to be placed within the <header>
    * before the >Hdr include segment
    */
	if (read_line(buf, inf))
   { /* empty file */
   	fprintf(stderr, "%s: input file is empty (%s)\n",
      	Prog, fname);
      fclose(inf);
      fclose(outf);
      return;
   }

   /* write title and whatever follows before blank line */
   fprintf(outf, "<html><head><title>\n%s\n</title>\n", buf);
   while (1)
   { /* loop copying lines until we find an empty one */
   	if (fgets(buf, BUFSIZ, inf) == NULL) break;
      if (*buf == '\0' || *buf == '\n' || *buf == '\r') break;	/* done */
      fputs(buf, outf);
   }

   /* now include Hdr segment, & process file */
   do_include(outf, "Hdr");
   bld_html(outf, inf);	/* process this file */
   fclose(inf);	/* close main album file */

   do_include(outf, "Ftr");	/* do footer */
}

void
bld_all_page()
{
	ProdAlphaList *lst;
   char *p;
   int i;

   if (check_dir()) return;

   if ((lst = list_files_in_directory(indir)) == NULL)
   {
   	fprintf(stderr, "%s: No files found in input directory!\n",
      	Prog);
      return;
   }

   for (i=0; i < lst->nitems; i++)
   { /* process all .html files */
   	if ((p = strstr(lst->item[i], ".html")) == NULL) continue;
      *p = '\0'; /* trim .html suffix */
      p = strrchr(lst->item[i], '/');
      if (p == NULL) p = strrchr(lst->item[i], '\\');
      if (p == NULL) p = lst->item[i];
   	bld_page(p);
   }
}

int
check_dir(void)
{
   if (indir == NULL) indir = ".";
   if (outdir == NULL) outdir = ".";
   if (strcmp(indir, outdir) == 0)
   { /* input and output directories can't be the same */
   	fprintf(stderr, "%s: Input & Output Directories must be different!\n",
      	Prog);
      return 1;
   }
   return 0;
}

